import SignetRequestController from '@/api/requests';
import {
  AuthenticationRequest,
  Bonus,
  ChangePasswordRequest,
  CloseRewardFundRequest,
  ContributeRequest,
  CreateQueueRequest,
  CreateQueueResponse,
  CreateRewardFundRequest,
  DistributeRewardsRequest,
  EditQueueRequest,
  EscalatePrivilegesRequest,
  GetBalanceRequest,
  GetBalanceResponse,
  GetContributionsRequest,
  GetContributionsResponse,
  GetQueueMembersRequest,
  GetQueueMembersResponse,
  GetQueuesResponse,
  GetRewardFundRequest,
  GetRewardFundResponse,
  GetRewardFundsRequest,
  GetRewardFundsResponse,
  GetUsersResponse,
  LoginResponse,
  NearlyCompleteFundsRequest,
  NearlyCompleteFundsResponse,
  Privileges,
  QueueMember,
  RewardDistributionInfo,
  SubmitRewardFundRequest,
  SuccessResponse,
} from '@/api/types';

const controller = new SignetRequestController();

export const usersExist = () => controller.post<SuccessResponse, null>('/UsersExist', null);

export const register = (username: string, password: string) => controller.post<SuccessResponse, AuthenticationRequest>('Register', {
  username,
  password,
});

export const login = (username: string, password: string) => controller.post<LoginResponse, AuthenticationRequest>('Login', {
  username,
  password,
});

export const getRewardFunds = (offset: number) => controller.post<GetRewardFundsResponse, GetRewardFundsRequest>('GetRewardFunds', { offset });

export const getQueues = () => controller.post<GetQueuesResponse, null>('GetQueues', null);

export const getQueueMembers = (id: number) => controller.post<GetQueueMembersResponse, GetQueueMembersRequest>('GetQueueMembers', { id });

export const createQueue = (name: string) => controller.post<CreateQueueResponse, CreateQueueRequest>('CreateQueue', { name });

export const createRewardFund = (
  asset: string,
  fundWallet: string,
  fundSecret: string,
  sellingWallet: string,
  issuerWallet: string,
  memo: string,
  minContribution: number,
  telegramLink: string,
  bonuses: Bonus[],
  queueID?: number | null | undefined,
) => controller.post<SuccessResponse, CreateRewardFundRequest>('CreateRewardFund', {
  asset,
  fundWallet,
  fundSecret,
  sellingWallet,
  issuerWallet,
  memo,
  minContribution,
  telegramLink,
  bonuses,
  queueID,
});

export const deleteRewardFund = (id: number, close: boolean) => controller.post<
  SuccessResponse, CloseRewardFundRequest
>(
  'CloseRewardFund',
  {
    id,
    close,
  },
);

export const getRewardFund = (id: number, consolidateContributions: boolean) => controller.post<GetRewardFundResponse, GetRewardFundRequest>('GetRewardFund', {
  id,
  consolidateContributions,
});

export const getContributions = (
  id: number,
  offset: number,
  forDate: string | undefined,
  consolidateContributions: boolean,
) => controller.post<GetContributionsResponse, GetContributionsRequest>(
  'GetContributions',
  {
    id,
    offset,
    forDate,
    consolidateContributions,
  },
);

export const getBalance = (secretKey: string) => controller.post<GetBalanceResponse, GetBalanceRequest>('GetBalance', { secretKey });

export const contribute = (privateKey: string, amount: number, rewardFund: number) => controller.post<SuccessResponse, ContributeRequest>('Contribute', {
  privateKey,
  amount,
  rewardFund,
});

export const getNearlyCompletedFunds = (threshold: number) => controller.post<NearlyCompleteFundsResponse, NearlyCompleteFundsRequest>('NearlyCompleteFunds', { threshold });

export const reorderQueue = (queueID: number, fundOrders: QueueMember[]) => controller.post<SuccessResponse, EditQueueRequest>('EditQueue', {
  queueID,
  fundOrders,
});

export const submitRewardFund = (fundID: number, submit: boolean) => controller.post<SuccessResponse, SubmitRewardFundRequest>('SubmitRewardFund', {
  fundID,
  submit,
});

export const distributeRewardFund = (rewardFundID: number, payments: RewardDistributionInfo[], distribute: boolean) => controller.post<SuccessResponse, DistributeRewardsRequest>('DistributeRewardFund', {
  rewardFundID,
  payments,
  distribute,
});

export const getUsers = () => controller.post<GetUsersResponse, null>('GetUsers', null);

export const changePrivileges = (userID: number, privileges: Privileges) => controller.post<SuccessResponse, EscalatePrivilegesRequest>('ChangePrivileges', {
  userID,
  privileges,
});

export const changePassword = (userID: number, password: string) => controller.post<SuccessResponse, ChangePasswordRequest>('ChangePassword', {
  userID,
  password,
});
