// eslint-disable-next-line no-shadow
import Decimal from 'decimal.js';
import { DateTime } from 'luxon';

// eslint-disable-next-line no-shadow
export enum Privileges {
  None = -1,
  SuperUser,
  AdminPlus,
  Admin
}

export interface Contribution {
  createdAt: string;
  amount: Decimal;
  rewardFundID: number;
  transactionID: string;
  wallet: string;
}

interface Contributions {
  list: Contribution[];
  dates: string[];
  total: number;
}

export interface RewardFund {
  id: number;
  asset: string;
  wallet: string;
  memo: string;
  amountGoal: number;
  minContribution: number;
  contributions: Contribution[] | null;
  telegramLink: string;
}

export interface Queue {
  id: number;
  name: string;
}

export interface QueueMember {
  id?: number;
  asset: string;
  order: number;
}

export interface CreateQueueRequest {
  name: string;
}

export interface CreateQueueResponse {
  id: number;
}

export interface GetQueuesResponse {
  queues: Queue[];
}

export interface SuccessResponse {
  success: boolean;
}

export interface GetRewardFundRequest {
  id: number;
  consolidateContributions: boolean;
}

export interface Bonus {
  goal?: number;
  percent?: number;
}

export interface CreateRewardFundRequest {
  asset: string;
  fundWallet: string;
  fundSecret: string;
  sellingWallet: string;
  issuerWallet: string;
  memo: string;
  minContribution: number;
  telegramLink: string;
  queueID?: number | null;
  bonuses: Bonus[];
}

export interface FundInfo {
  id: number;
  asset: string;
  fundWallet: string;
  sellingWallet: string;
  issuerWallet: string;
  memo: string;
  price: number;
  amountAvailable: number;
  amountGoal: number;
  minContribution: number;
  telegramLink: string;
  bonuses: Bonus[];
  queueID: number | null;
}

interface Total {
  amountHeld: number;
}

export interface GetRewardFundResponse {
  fundInfo: FundInfo;
  contributions: Contributions;
  total: Total;
}

export interface GetBalanceRequest {
  secretKey: string;
}

export interface GetBalanceResponse {
  balance: number;
}

export interface ContributeRequest {
  privateKey: string;
  amount: number;
  rewardFund: number;
}

export interface AuthenticationRequest {
  username: string;
  password: string;
}

export interface LoginResponse {
  token: string | null;
  lastLogin: DateTime | null;
}

export interface GetQueueMembersRequest {
  id: number;
}

export interface GetQueueMembersResponse {
  members: RewardFund[];
}

export interface GetRewardFundsRequest {
  offset: number;
}

export interface GetRewardFundsResponse {
  rewardFunds: FundInfo[];
  total: number;
}

export interface Claims {
  id: number;
  username: string;
  privileges: Privileges;
  exp: number;
}

export interface NearlyCompletedFund {
  id: number;
  asset: string;
  minContribution: number;
  amountAvailable: number;
  memo: string;
  fundWallet: string;
  raised: number;
}

export interface NearlyCompleteFundsRequest {
  threshold: number;
}

export interface NearlyCompleteFundsResponse {
  funds: NearlyCompletedFund[];
}

export interface GetContributionsRequest {
  id: number;
  offset: number;
  forDate: string | undefined;
  consolidateContributions: boolean;
}

export type GetContributionsResponse = Contributions;

export interface CloseRewardFundRequest {
  id: number;
  close: boolean;
}

export interface EditQueueRequest {
  queueID: number;
  fundOrders: QueueMember[];
}

export interface SubmitRewardFundRequest {
  fundID: number;
  submit: boolean;
}

export interface RewardDistributionInfo {
  destination: string;
  amount: number;
}

export interface DistributeRewardsRequest {
  rewardFundID: number;
  payments: RewardDistributionInfo[];
  distribute: boolean;
}

export interface User {
  id: number;
  username: string;
  password: string;
  admin: number;
}

export interface GetUsersResponse {
  users: User[];
}

export interface EscalatePrivilegesRequest {
  userID: number;
  privileges: Privileges;
}

export interface ChangePasswordRequest {
  userID: number;
  password: string;
}
