import {
  createRouter,
  createWebHistory,
  RouteRecordRaw,
} from 'vue-router';
import RegisterView from '@/views/RegisterView.vue';
import LoginView from '@/views/LoginView.vue';
import { Privileges } from '@/api/types';
import HomeView from '@/views/HomeView.vue';
import FundView from '@/views/FundView.vue';
import AddFundView from '@/views/AddFundView.vue';
import AdminView from '@/views/AdminView.vue';
import ModifyQueueView from '@/views/ModifyQueueView.vue';
import AdminDashboardView from '@/views/AdminDashboardView.vue';
import ModifyUserView from '@/views/ModifyUserView.vue';
import ChangePasswordView from '@/views/ChangePasswordView.vue';
import store from '@/store';
import ForbiddenView from '@/views/ForbiddenView.vue';

const routes: Array<RouteRecordRaw> = [
  {
    path: '/',
    name: 'home',
    component: HomeView,
    meta: { title: 'Home' },
  },
  {
    path: '/fund/:id',
    name: 'fund',
    component: FundView,
    meta: { title: undefined },
  },
  {
    path: '/login',
    name: 'login',
    component: LoginView,
    meta: { title: 'Login' },
  },
  {
    path: '/changepassword',
    name: 'changepassword',
    component: ChangePasswordView,
    meta: { title: 'Change Password' },
  },
  {
    path: '/admin',
    name: 'admin',
    component: AdminView,
    redirect: '/admin/dashboard',
    children: [
      {
        path: 'dashboard',
        name: 'admindashboard',
        component: AdminDashboardView,
        meta: {
          requiredRights: Privileges.Admin,
          title: 'Admin',
        },
      },
      {
        path: 'addfund',
        name: 'addfund',
        component: AddFundView,
        meta: {
          requiredRights: Privileges.Admin,
          title: 'Add Group Fund',
        },
      },
      {
        path: 'modifyfund',
        name: 'modifyfund',
        component: AddFundView,
        meta: {
          requiredRights: Privileges.Admin,
          title: 'Add Group Fund',
        },
      },
      {
        path: 'addqueue',
        name: 'addqueue',
        component: AddFundView,
        meta: {
          requiredRights: Privileges.Admin,
          title: 'Add Group Fund',
        },
      },
      {
        path: 'modifyqueue',
        name: 'modifyqueue',
        component: ModifyQueueView,
        meta: {
          requiredRights: Privileges.Admin,
          title: 'Add Group Fund',
        },
      },
      {
        path: 'adduser',
        name: 'adduser',
        component: RegisterView,
        meta: {
          requiredRights: Privileges.AdminPlus,
          title: 'Add User',
        },
      },
      {
        path: 'modifyuser',
        name: 'modifyuser',
        component: ModifyUserView,
        meta: {
          requiredRights: Privileges.AdminPlus,
          title: 'Modify User',
        },
      },
    ],
    meta: {
      requiredRights: Privileges.Admin,
      title: 'Administrator',
    },
  },
  {
    path: '/forbidden',
    name: 'forbidden',
    component: ForbiddenView,
  },
  {
    path: '/:pathMatch(.*)*',
    name: 'wildcard',
    redirect: '/',
  },
];

const router = createRouter({
  history: createWebHistory(process.env.BASE_URL),
  routes,
});

router.beforeEach(async (to, from, next) => {
  document.title = `Beignet - ${to.meta.title}`;

  if (!store.getters.passwordChangeRequired) {
    return next();
  }
  if (to.name !== 'changepassword') {
    return next('changepassword');
  }
  return next(undefined);
});

export default router;
